/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "TransportNode.h"
#include <jack/transport.h>
#include "InputPort.h"
#include "OutputPort.h"
#include "Plugin.h"
#include "JackDriver.h"
#include "Port.h"
#include "util.h"
#include "Om.h"

namespace Om {


TransportNode::TransportNode(const string& path, uint poly, Patch* parent, samplerate srate, size_t buffer_size)
: InternalNode(path, 1, parent, srate, buffer_size)
{
	m_num_ports = 1;
	m_ports.alloc(m_num_ports);

	// Add s/beat output port
	OutputPort* spb_port = new OutputPort(this, "s-per-beat", 1, 1,
		new PortInfo("s-per-beat", CONTROL, OUTPUT, 0, 0, 1), m_buffer_size);
	m_ports.at(0) = spb_port;

	m_plugin.type(Plugin::Internal);
	m_plugin.lib_path("");
	m_plugin.lib_name("");
	m_plugin.plug_label("transport");
	m_plugin.name("Om transport node");
}


void
TransportNode::run(size_t nframes)
{
	NodeBase::run(nframes);

	double bpm = om->jack_driver()->position()->beats_per_minute;
	float spb = 60.0 / bpm;
	
	//cerr << "bpm = " << bpm << endl;
	//cerr << "spb = " << spb << endl; 

	m_ports.at(0)->set_value(0, spb);
}


} // namespace Om

